import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:slick/configuration/slick_configuration.dart';
import 'package:slick/controls/form_input/slick_text_input.dart';
import 'package:intl/intl.dart';

enum SlickDateMode { Date, TimeOfDay, DateAndTime }

class SlickDateInput extends StatefulWidget {
  SlickDateInput({
    Key key,
    this.initialDateValue,
    this.labelText,
    this.onDateChanged,
    this.onTimeOfDayChanged,
    this.mode = SlickDateMode.Date,
    this.readonly = false,
    this.initialTimeOfDayValue,
  }) : super(key: key);

  final DateTime initialDateValue;
  final TimeOfDay initialTimeOfDayValue;
  final String labelText;
  final ValueChanged<DateTime> onDateChanged;
  final ValueChanged<TimeOfDay> onTimeOfDayChanged;
  final bool readonly;
  final SlickDateMode mode;

  @override
  _SlickDateInputState createState() => _SlickDateInputState();
}

class _SlickDateInputState extends State<SlickDateInput> {
  final TextEditingController _controller = TextEditingController();
  DateTime _currentDate;

  @override
  void initState() {
    if (widget.initialDateValue != null) {
      _currentDate = widget.initialDateValue;
    }

    _currentDate ??= DateTime.now();

    if (widget.initialTimeOfDayValue != null) {
      _currentDate = DateTime(
        _currentDate.year,
        _currentDate.month,
        _currentDate.day,
        widget.initialTimeOfDayValue.hour,
        widget.initialTimeOfDayValue.minute,
        _currentDate.second,
        _currentDate.millisecond,
        _currentDate.microsecond,
      );
    }
    _controller.text = getValue(_currentDate);

    _controller.addListener(() {
      fireOnChanged(_controller.text);
    });

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlickTextInput(
      readOnly: widget.readonly,
      labelText: widget.labelText,
      controller: _controller,
      suffix: Row(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          if (widget.mode == SlickDateMode.Date ||
              widget.mode == SlickDateMode.DateAndTime)
            Builder(
              builder: (context) => MouseRegion(
                cursor: SystemMouseCursors.click,
                child: GestureDetector(
                  child: Icon(
                    Icons.date_range_rounded,
                    color: SlickConfiguration.colors.inputControlBorder,
                    size: 21,
                  ),
                  onTap: () async {
                    final picked = await showDatePicker(
                        context: context,
                        initialDate: _currentDate,
                        initialDatePickerMode: DatePickerMode.day,
                        firstDate: DateTime(1900),
                        lastDate: DateTime(2030));
                    if (picked != null) {
                      setState(() {
                        _currentDate = DateTime(
                          picked.year,
                          picked.month,
                          picked.day,
                          _currentDate.hour,
                          _currentDate.minute,
                          _currentDate.second,
                          _currentDate.microsecond,
                          _currentDate.microsecond,
                        );
                        _controller.text = getValue(_currentDate);
                        fireOnChanged(_controller.text);
                      });
                    }
                  },
                ),
              ),
            ),
          if (widget.mode == SlickDateMode.DateAndTime)
            SizedBox(
              width: 4.0,
            ),
          if (widget.mode == SlickDateMode.DateAndTime ||
              widget.mode == SlickDateMode.TimeOfDay)
            Builder(
              builder: (context) => MouseRegion(
                cursor: SystemMouseCursors.click,
                child: GestureDetector(
                  child: Icon(
                    Icons.access_time_rounded,
                    color: SlickConfiguration.colors.inputControlBorder,
                    size: 21,
                  ),
                  onTap: () async {
                    final picked = await showTimePicker(
                      context: context,
                      initialTime: TimeOfDay(
                        hour: _currentDate.hour,
                        minute: _currentDate.minute,
                      ),
                    );
                    if (picked != null) {
                      setState(() {
                        _currentDate = DateTime(
                          _currentDate.year,
                          _currentDate.month,
                          _currentDate.day,
                          picked.hour,
                          picked.minute,
                          _currentDate.second,
                          _currentDate.millisecond,
                          _currentDate.microsecond,
                        );
                        _controller.text = getValue(_currentDate);
                        fireOnChanged(_controller.text);
                      });
                    }
                  },
                ),
              ),
            ),
          SizedBox(
            width: 4.0,
          ),
        ],
      ),
    );
  }

  DateTime fireOnChanged(String val) {
    if (val.isEmpty) widget.onDateChanged(null);
    var dateVal = DateTime.tryParse(val);
    dateVal ??= widget.initialDateValue;
    if (widget.onDateChanged != null) {
      widget.onDateChanged(dateVal);
    }

    if (widget.onTimeOfDayChanged != null) {
      widget.onTimeOfDayChanged(
        TimeOfDay(hour: dateVal.hour, minute: dateVal.minute),
      );
    }
    return dateVal;
  }

  String getValue(DateTime dateValue) {
    if (dateValue == null) return '';

    switch (widget.mode) {
      case SlickDateMode.Date:
        return DateFormat('yyyy-MM-dd').format(dateValue);
      case SlickDateMode.TimeOfDay:
        return DateFormat('HH:mm').format(dateValue);
      case SlickDateMode.DateAndTime:
        return DateFormat('yyyy-MM-dd HH:mm').format(dateValue);
      default:
        return '';
    }
  }
}
